@@ -24,10 +24,12 @@ package.skeleton.dx <- structure(function # Package skeleton deluxe
  excludePattern=FALSE,
 ### A regular expression matching the files that are not to be processed e.g. because 
 ### inlinedocs can not handle them yet (like generic function definitions)
+# codefiles,
  ...
 ### Parameters to pass to Parser Functions.
  ){
-  ##alias<< inlinedocs	 
+  ## This causes a warning on R CMD check TDH 28 Jan 2013.
+  ##alias< < inlinedocs
 	 
   chdir <- file.path(pkgdir,"R")
   if(!file.exists(chdir))stop("need pkgdir/R, tried ",chdir)
@@ -160,7 +162,21 @@ package.skeleton.dx <- structure(function # Package skeleton deluxe
   code_files <- if(!"Collate"%in%colnames(desc))Sys.glob("*.R")
   else strsplit(gsub("\\s+"," ",desc[,"Collate"]),split=" ")[[1]]
   code_files =grep(excludePattern,code_files,invert=TRUE,value=TRUE)
-  ##print(excludePattern)
+  ## TDH 28 Jan 2013, warn users such as Pierre Neuvial if they have
+  ## comments on the last line of one input file. Sometimes comments
+  ## on the last line can appear to be the first line of comments of
+  ## the next code file.
+  lines.list <- lapply(code_files,readLines)
+  for(i in seq_along(lines.list)){
+    lvec <- lines.list[[i]]
+    fn <- code_files[i]
+    last <- lvec[length(lvec)]
+    if(grepl("#",last)){
+      warning("comment on last line of ",fn,
+              ", unexpected docs may be extracted")
+    }
+  }
+  #print(excludePattern)
   ## Make package skeleton and edit Rd files (eventually just don't
   ## use package.skeleton at all?)
   name <- desc[,"Package"]
@@ -183,7 +199,12 @@ package.skeleton.dx <- structure(function # Package skeleton deluxe
   if ( any(dup.names) ){
     warning("duplicated file names in docs: ",paste(names(docs)[dup.names]))
   }
-  for(N in unique(names(docs))) modify.Rd.file(N,name,docs)
+  for(N in unique(names(docs))) {
+	modify.Rd.file(N,name,docs)
+        if(grepl("class",N)){
+		removeAliasesfrom.Rd.file(N,name,code)
+	}
+  }
   file.copy(file.path(name,'man'),"..",recursive=TRUE)
   # PhG: copy NAMESPACE file back
   if (isTRUE(namespace)) {
@@ -298,6 +319,100 @@ replace.one <- function
   }
   txt
 }
+removeAliasesfrom.Rd.file<- function
+### remove aliases to methodnames from the Rd file of a class
+### automatically-generated by package.skeleton.
+(N,
+### Name of function/file to which we will add documentation.
+ pkg,
+### Package name.
+code
+### The code of the package 
+)
+{
+  # <mm:package.skeleton adds some duplicated aliases to the .*-class.Rd files 
+  # to get rid of the warnings from R CMD check 
+  # we have to delete them 
+  # the duplicated aliases have 2 sources
+
+  Nme <- utils:::.fixPackageFileNames(N)
+  fb <- paste(Nme,".Rd",sep="")
+  f <- file.path(pkg,'man',fb)
+
+  ## If we do.not.generate this file, it does not exist so we need to
+  ## do nothing to avoid errors.
+  if(!file.exists(f))return()
+  
+  dlines <- readLines(f)
+  
+  name=gsub("-class","",N)
+  
+  # these we will later comment out
+  # now we look at all the aliases produces by package.skeleton
+  aliasLine2name=function(line){return(gsub("^\\\\alias\\{(.*)\\}","\\1",line))}
+  aliasInd <- grep("^\\\\alias.*",dlines)
+  # aliasnames=as.character(lapply(dlines[aliasInd],aliasLine2name))
+  # duplicates= intersect(methodnames,aliasnames)
+  
+  if(length(aliasInd)){
+      # first get rid of the ${classname}-mehthod stuff
+      p1=paste(",",name,"-method",sep="")
+  
+      # next get rid of the -mehthod stuff
+      patterns=c(p1,",ANY-method",",ANY",",character")
+      for (pattern in patterns){
+          dlines[aliasInd] <- gsub(pattern,"",dlines[aliasInd]) 
+      }
+  }
+  # next: 
+  # the names of the methods implemented by the class
+  # which occur in all ".*-class.Rd" files 
+  # as aliases 
+  # to find them we ask for those methods but 
+  # therefore have to readthe the code to be documented.
+  # As the apply.parsers function we do this in a separate 
+  # environment
+  # mm:
+  # This could probably be factored out since it duplicates
+  # some functionality of apply.parsers (evaluating the sources again)
+  # but I am not sure how big the changes involved woud be.
+  
+  e <- new.env()
+  old <- options(keep.source=TRUE,topLevelEnvironment=e)
+  on.exit(options(old))
+  exprs <- parse(text=code)
+  ## set this so that no warnings about creating a fake
+  ## package when we try to process S4 classes defined in code
+  e$.packageName <- "inlinedocs.processor"
+  for (i in exprs){
+      eval(i, e)
+  }
+  g=file(,open="w+")#anonymous file
+  showMethods(classes=name,printTo=g,where=e)
+  lines=readLines(g)
+  close(g)
+  ind=grep("Function",lines)
+  flines=lines[ind]
+  line2name=function(line){return(strsplit(line," ")[[1]][2])}
+  methodnames=as.character(lapply(flines,line2name))
+  markDupAliases=function(line){
+  	ret=line
+  	if(is.element(aliasLine2name(line),methodnames)){
+  		ret=gsub("(^\\\\alias)","%% \\1",line)
+  	}
+  	return(ret)
+  } 
+  for (j in aliasInd){
+  	dlines[[j]]=markDupAliases(dlines[[j]])
+  }
+  #print(fb)
+  #print(d)
+  #print(dlines)
+  txt <- paste(dlines,collapse="\n")
+  fc=file(f,open="w+")#anonymous file
+  writeLines(txt,fc)
+  close(fc)
+}	
 
 modify.Rd.file <- function
 ### Add inline documentation from comments to an Rd file
@@ -329,16 +444,26 @@ modify.Rd.file <- function
   ## it's easy to get started.
   if((length(docs[[N]])<3) &&
      file.exists(file.path("..","man",fb))){
+    # print(paste("mm object with no documentation available N=",N))
     unlink(f)
     return()
   }
   cat(N,":",sep="")
   d <- docs[[N]]
+  ## for some functions no documentatian file is created by package.skeleton
+  ## for instance generic functions that are already defined in other packages
+  ## like print or plot so there is still the possibillity that 
+  ## f is missing altogether
+  if (!file.exists(f)) {
+	print("mm missing file")
+	print(f)
+	return()
+	}
   dlines <- readLines(f)
 
   ## cut out alias line if we are in the package file and there is a
   ## matching function
-  if(length(grep("-package$",N)))
+  if(length(grep("-package$",N)) && "alias" %in% names(d) )
     dlines <- dlines[-grep(paste("alias[{]",N,sep=""),dlines)-1]
   else if ( "alias" %in% names(d) ){
     ## allowing alias changes have to make sure that original alias remains
@@ -347,6 +472,7 @@ modify.Rd.file <- function
     d[["alias"]] <- paste(paste(N,"}\n\\alias{",sep=""),
                             d[["alias"]],sep="")
   }
+  
 
   # PhG: in the special case of custom operators like %....%, we must protect
   # these strings in name, alias and usage (at least)! Otherwise, bad things
@@ -397,7 +523,7 @@ modify.Rd.file <- function
   if(length(i)){
     dlines[i] <- gsub("\\\\code[{][^}]*[}]","",dlines[i])
   }
-  
+  name=N
   txt <- paste(dlines,collapse="\n")
   
   ## Fix usage
@@ -405,15 +531,37 @@ modify.Rd.file <- function
   Mend <- m+attr(m,"match.length")
   utxt <- substr(txt,m+6,Mend-2)
 
+  ## fix \method version if .s3method
+  if ( is.null(d$.s3method) ) {
+	  # PhG: in case we have fun<-(x, ..., value), we must rewrite it
+	  # as fun(x, ...) <- value
+	  if (grepl("<-$", N)) {
+		  utxt <- sub("<-[(](.+), ([^,)]+)[)]",
+				  "(\\1) <- \\2", utxt)
+	  }
+	  # PhG: this is for special functions %...% which should write x %...% y
+	  if (grepl("^%.*%$", N)) {
+		  utxt <- sub("(%.*%)[(]([^,]+), ([^)]+)[)]",
+				  "\\2 \\1 \\3", utxt) 
+	  }
+  }
+  
   ## multiple lines for the PDF!
-  parsed <- parse(text=utxt)
+  # tw: parse fails on accessor functions such as "myF<-" <- function(data,x) 
+  # see testfile accessorFunctions.R
+  # workaround with tryCatch
+  parsed <- utxt
+  tryCatch({
+	  parsed <- parse(text=utxt)
+  }, error = function(e) warning(e) )
   if(length(parsed)){
-    utxt <- sprintf("usage{%s}\n",paste(format(parsed[[1]]),collapse="\n"))
+	  utxt <- sprintf("usage{%s}\n",paste(format(parsed[[1]]),collapse="\n"))
   }
   if(length(grep("usage[{]data",utxt))){
     utxt <- gsub("data[(]([^)]*)[)]","\\1",utxt)
   }
   ## fix \method version if .s3method
+
   if ( !is.null(d$.s3method) ){
     pat <- paste(d$.s3method,collapse=".")
     rep <- paste("\\method{xx",d$.s3method[1],"}{",d$.s3method[2],"}",sep="")
@@ -428,17 +576,7 @@ modify.Rd.file <- function
         utxt <- sub(", *([^),]+)[)]", ") <- \\1", utxt)
     }
   } else {
-    # PhG: in case we have fun<-(x, ..., value), we must rewrite it
-    # as fun(x, ...) <- value
-    if (grepl("<-$", N)) {
-        utxt <- sub("<-[(](.+), ([^,)]+)[)]",
-            "(\\1) <- \\2", utxt)
-    }
-    # PhG: this is for special functions %...% which should write x %...% y
-    if (grepl("^%.*%$", N)) {
-        utxt <- sub("(%.*%)[(]([^,]+), ([^)]+)[)]",
-            "\\2 \\1 \\3", utxt) 
-    }
+	#tw: moved before parse
   }
   ## add another backslash due to bug in package.skeleton
   ## but only if not before % character due to another bug if % in usage
@@ -483,4 +621,3 @@ modify.Rd.file <- function
   cat(txt,file=f)
   cat("\n")
 }
-
